var PersInfo = function()
{
	var persinfo = {};
	var debug = 0;
	
	var patientInfo = null;
	var caseInfo = null;
	var bloodInfo = null;

	var pnrIsValid = false;
	var pnrIsAcceptable = false;
	var caseIsValid = false;

	var nofTubesIsValid = false;
	var samplingDateIsValid = true;

	var wellIsValid = [];

	persinfo.initPage = function()
	{
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', persinfo.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.focusOnEnter);
		Events.addEventHandler('caseName', 'blur', persinfo.caseNameOnChange);
		Events.addEventHandler('personalNumber', 'keyup', persinfo.personalNumberOnChange);
		Events.addEventHandler('personalNumber', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', persinfo.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', persinfo.validateStep2);
		Events.addEventHandler('familyName', 'blur', persinfo.nameOnChange);
		Events.addEventHandler('allFirstNames', 'blur', persinfo.nameOnChange);
		Events.addEventHandler('allFirstNames', 'keypress', Wizard.goNextOnTabOrEnter);

		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', persinfo.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', persinfo.validateStep3);
		Events.addEventHandler('plNumber', 'blur', persinfo.plNumberOnChange);
		Events.addEventHandler('pad', 'blur', persinfo.padOnChange);
		Events.addEventHandler('nofTubes', 'keypress', Events.integerOnly);
		Events.addEventHandler('nofTubes', 'blur', persinfo.nofTubesOnChange);
		Events.addEventHandler('arrivalDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('samplingDate', 'blur', persinfo.samplingDateOnChange);
		Events.addEventHandler('mutationAnalysisKrasNras', 'click', persinfo.mutationAnalysisKrasNrasOnChange);
		Events.addEventHandler('mutationAnalysisBraf', 'click', persinfo.mutationAnalysisBrafNrasOnChange);
		Events.addEventHandler('mutationAnalysisEgfr', 'click', persinfo.mutationAnalysisEgfrOnChange);
		Events.addEventHandler('site', 'change', persinfo.siteOnChange);
		Events.addEventHandler('specimenType', 'change', persinfo.specimenTypeOnChange);
		Events.addEventHandler('projectFocus', 'change', persinfo.projectFocusOnChange);
		Events.addEventHandler('otherPathNote', 'keypress', Wizard.goNextOnTab);
		
		// Step 4
		Events.addEventHandler('step-4', 'wizard-initialize', persinfo.initializeStep4);
		Events.addEventHandler('step-5', 'wizard-validate', persinfo.validateStep4);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goupdate', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', persinfo.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	persinfo.caseNameOnChange = function()
	{
		var frm = document.forms['meludi'];
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		if (!Meludi.isValidCaseName(caseName))
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Only "ML" + 5 digits is allowed.');
			frm.caseName.focus();
			return;
		}

		frm.caseName.value = caseName;
		Wizard.setInputStatus('caseName', 'valid');
		caseIsValid = true;
		
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		Wizard.showLoadingAnimation('Loading information about case...');
		Wizard.asyncJsonRequest(url, persinfo.onCaseInfoLoaded);
	}

	persinfo.onCaseInfoLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		
		// Get biosource information from the AJAX response
		caseInfo = response.caseInfo;
		
		if (caseInfo && caseInfo.patient)
		{
			frm.personalNumber.value = caseInfo.patient.personalNumber;
			persinfo.personalNumberOnChange();
			Wizard.goNext(false);
		}
	}

	persinfo.personalNumberOnChange = function()
	{
		var frm = document.forms['meludi'];
		var pnr = frm.personalNumber.value;
		pnrIsValid = false;
		pnrIsAcceptable = true;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('personalNumber', 'warning', pnrLen < 12 ? 'Too short' : 'Too long');
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Non-standard personal number');
			return;
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Not a valid date');
			return;
		}
		
		if (!Meludi.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Invalid control digit');
			return;
		}

		Wizard.setInputStatus('personalNumber', 'valid');
		pnrIsValid = true;
		Wizard.hideGoNextConfirmation();
	}

	persinfo.validateStep1 = function(event)
	{
		if (event.detail.auto)
		{
			if (!caseIsValid || !pnrIsValid) 
			{
				if (pnrIsAcceptable)
				{
					var frm = document.forms['meludi'];
					frm.personalNumber.focus();
					Wizard.showGoNextConfirmation(true, 'Check to verify registration of non-standard personal number.');
				}
				event.preventDefault();
			}
		}
		else
		{
			if (!caseIsValid || !pnrIsAcceptable) event.preventDefault();
		}
	}

	persinfo.initializeStep2 = function(event)
	{
		var frm = document.forms['meludi'];
		var pnr = frm.personalNumber.value;
		if (pnr.length == 10 && pnrIsValid) 
		{
			var year = Meludi.fullYear(parseInt(pnr.substr(0, 2), 10));
			pnr = year + pnr.substr(2);
		}
	
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetPatientInfo';
		url += '&personalNumber=' + encodeURIComponent(pnr);
		url += '&pnrIsValid=' + pnrIsValid;

		Wizard.showLoadingAnimation('Loading patient information...');
		Wizard.asyncJsonRequest(url, persinfo.patientInfoLoaded);
	}

	
	persinfo.patientInfoLoaded = function(response)
	{
		Wizard.setCurrentStep(2);
		// Get patient information from the AJAX response
		patientInfo = response.patientInfo;
		var frm = document.forms['meludi'];


		frm.patientCode.value=patientInfo.name;
		frm.familyName.value = patientInfo.familyName || '';
		frm.allFirstNames.value = patientInfo.allFirstNames || '';
		Doc.element('dateOfBirth').innerHTML = patientInfo.dateOfBirth ? Strings.encodeTags(patientInfo.dateOfBirth) : '---';
		Doc.element('gender').innerHTML = patientInfo.gender ? Strings.encodeTags(patientInfo.gender) : '---';
		frm.familyName.focus();

		if (!patientInfo.id)
		{
			if (!pnrIsValid)
			{
				Wizard.setInputStatus('gender', 'warning', 'Gender and DateOfBirth can\'t be automatically generated for non-standard Personal number');
			}
		}
		else
		{
			Doc.element('step-2-title').innerHTML = 'Existing patient: Verify names';
			frm.patientCode.disabled = true;
		}
		
		Doc.show('gonext');
		Doc.show('gocancel');
		
	}
	
	persinfo.nameOnChange = function(event)
	{
		var field = event.target;
		patientInfo[field.name] = field.value;
		if (field.value == '')
		{
			Wizard.setInputStatus(field, 'warning', 'Missing');
		}
		else
		{
			Wizard.setInputStatus(field, 'valid');
			Wizard.hideGoNextConfirmation();
		}
	}

	persinfo.validateStep2 = function(event)
	{
		var valid = true;
		var acceptable = true;
		var frm = document.forms['meludi'];

		if (frm.allFirstNames.value == '')
		{
			Wizard.setInputStatus('allFirstNames', 'warning', 'Missing');
			valid = false;
		}
		else
		{
			Wizard.setInputStatus('allFirstNames', 'valid');
		}
		
		if (frm.familyName.value == '')
		{
			Wizard.setInputStatus('familyName', 'warning', 'Missing');
			valid = false;
		}
		else
		{
			Wizard.setInputStatus('familyName', 'valid');
		}

		// New patient only
		if (!patientInfo.id)
		{
			if (frm.patientCode.value == '')
			{
				Wizard.setInputStatus('patientCode', 'invalid', 'Missing');
				frm.patientCode.focus();
				valid = false;
				acceptable = false;
			}
			else
			{
				Wizard.setInputStatus('patientCode', 'valid');
				patientInfo.name = frm.patientCode.value;
			}	
		}
		
		if (event.detail.auto)
		{
			if (!valid)
			{
				Wizard.showGoNextConfirmation(true, 'Check to verify registration with missing name(s).');
				event.preventDefault();
			}
		}
		else
		{
			if (!acceptable) event.preventDefault();
		}
		
	}
	
	persinfo.initializeStep3 = function()
	{
		// Check entered case and pnr with AJAX
		var frm = document.forms['meludi'];
		var updateMode = caseInfo.id && caseInfo.patient;
		
		// Generate list of specimen tubes
		var hasUnknownSpecimen = false;
		var hasSpecimen = caseInfo.specimen && caseInfo.specimen.length > 0;
		var hasDna = caseInfo.dna && caseInfo.dna.length > 0;
		var hasRna = caseInfo.rna && caseInfo.rna.length > 0;
		var thisCasePlNumber = null;
		var hasPlNumber = false;
		var thisCasePAD = null;
		var hasPAD = false;
		var thisCaseSamplingDate = null;
		var thisCaseExtractionDate = null;
		var thisCaseTubeContentType = caseInfo.tubeContentType;
		var thisCaseSpecimenType = null;

		if (hasSpecimen)
		{
			var specimenTubes = '';
			for (var i = 0; i < caseInfo.specimen.length; i++)
			{
				var specimen = caseInfo.specimen[i];
				specimenTubes += Strings.encodeTags(specimen.name);
				if (specimen.plNumber && !hasPlNumber) 
				{
					hasPlNumber = true;
					thisCasePlNumber = specimen.plNumber;
				}
				if (specimen.pad && !hasPAD) 
				{
					hasPAD = true;
					thisCasePAD = specimen.pad;
				}
				if (specimen.samplingDate && !thisCaseSamplingDate) 
				{
					thisCaseSamplingDate = specimen.samplingDate;
				}
				if (specimen.specimenType && !thisCaseSpecimenType) 
				{
					thisCaseSpecimenType = specimen.specimenType;
				}
				specimenTubes += '<br>';
			}
			Doc.element('tubeContentType').value = thisCaseTubeContentType;
			Doc.element('specimenType').value = thisCaseSpecimenType;

			Doc.show('specimenTubesSection');
			Doc.element('specimenTubes').innerHTML = specimenTubes;

			Doc.hide('nofTubesSection');
			Doc.hide('arrivalDateSection');

			Doc.show('specimenTubesSection');
			Doc.show('samplingDateSection');
			
			Doc.hide('mutationAnalysisSection');
			Doc.hide('siteSection');
			Doc.hide('sampleTypeSection');
			Doc.hide('projectFocusSection');
			Doc.hide('otherPathNoteSection');

			// Populate 'Sampling date' field
			if (thisCaseSamplingDate != null)
			{
				frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
				persinfo.samplingDateTimeOnChange();
			}
		
			frm.plNumber.value = thisCasePlNumber;
			frm.pad.value = thisCasePAD;

			if (caseInfo.id != null && typeof caseInfo.id != 'undefined')
			{
				frm.samplingDate.disabled = true;
				frm.tubeContentType.disabled = true;
				frm.specimenType.disabled = true;
			}

			// Existing cases for this patient
			var hasUnknownCase = false;
			if (patientInfo.cases && patientInfo.cases.length > 0)
			{
				var cases = '';
			
				if (patientInfo.cases.length == 1 || updateMode)
				{
					if (!updateMode)
					{
						cases += '[<i>new case</i>]<br>';
					}				
				}
			
				if (updateMode)
				{
					if (caseInfo.merged)
					{
						Doc.element('step-3-title').innerHTML = 'Update case \''+Strings.encodeTags(caseInfo.originalName)+'\' (which is merged with case \'' + Strings.encodeTags(caseInfo.name) + '\')';
					}
					else
					{
						Doc.element('step-3-title').innerHTML = 'Update case \''+Strings.encodeTags(caseInfo.name)+'\'';
					}
				
				}
				else if (patientInfo.cases.length == 2)
				{
					Doc.element('step-3-title').innerHTML = 'Merge with existing case';	
				}
				else if (patientInfo.cases.length > 0)
				{
					Doc.element('step-3-title').innerHTML = 'Create new case or merge with existing case';
				}
			}

			Wizard.setCurrentStep(3);
			if (updateMode)
			{
				Doc.show('goupdate');
			}
			else
			{
				Doc.show('goregister');
			}
			Doc.show('gocancel');
			frm.plNumber.focus();
		}
		else if (hasDna)
		{
			var dnaTubes = '';
			for (var i = 0; i < caseInfo.dna.length; i++)
			{
				var dna = caseInfo.dna[i];
				dnaTubes += Strings.encodeTags(dna.name);
				if (dna.extractionDate && !thisCaseExtractionDate) 
				{
					thisCaseExtractionDate = dna.extractionDate;
				}
				dnaTubes += '<br>';
			}
			Doc.element('tubeContentType').value = thisCaseTubeContentType;

			frm.plNumber.disabled = true;
			Doc.element('plNumberMessageText').innerHTML = "No PL-number can be saved for case without specimen.";
			frm.pad.disabled = true;
			Doc.element('padMessageText').innerHTML = "No PAD/CL number can be saved for case without specimen.";

			Doc.show('specimenTubesSection');
			Doc.element('specimenTubesTitle').innerHTML = "DNA tubes";
			Doc.element('specimenTubes').innerHTML = dnaTubes;
			Doc.element('specimenTubesMessageText').innerHTML = "The DNA tube(s) associated with this case.";

			Doc.hide('nofTubesSection');
			Doc.hide('arrivalDateSection');

			Doc.show('specimenTubesSection');
			Doc.show('samplingDateSection');
			
			Doc.hide('mutationAnalysisSection');
			Doc.hide('siteSection');
			Doc.hide('sampleTypeSection');
			Doc.hide('projectFocusSection');
			Doc.hide('otherPathNoteSection');

			thisCaseSamplingDate = thisCaseExtractionDate;
			// Populate 'Sampling date' field
			if (thisCaseSamplingDate != null)
			{
				frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
				persinfo.samplingDateTimeOnChange();
			}
		
			frm.plNumber.value = thisCasePlNumber;
			frm.pad.value = thisCasePAD;

			if (caseInfo.id != null && typeof caseInfo.id != 'undefined')
			{
				frm.samplingDate.disabled = true;
				frm.tubeContentType.disabled = true;
				frm.specimenType.disabled = true;
			}

			Wizard.setCurrentStep(3);
			if (updateMode)
			{
				Doc.show('goupdate');
			}
			else
			{
				Doc.show('goregister');
			}
			Doc.show('gocancel');
			frm.plNumber.focus();
		}
		else if (hasRna)
		{
			var rnaTubes = '';
			for (var i = 0; i < caseInfo.rna.length; i++)
			{
				var rna = caseInfo.rna[i];
				rnaTubes += Strings.encodeTags(rna.name);
				if (rna.extractionDate && !thisCaseExtractionDate) 
				{
					thisCaseExtractionDate = rna.extractionDate;
				}
				rnaTubes += '<br>';
			}
			Doc.element('tubeContentType').value = thisCaseTubeContentType;

			frm.plNumber.disabled = true;
			Doc.element('plNumberMessageText').innerHTML = "No PL-number can be saved for case without specimen.";
			frm.pad.disabled = true;
			Doc.element('padMessageText').innerHTML = "No PAD/CL number can be saved for case without specimen.";

			Doc.show('specimenTubesSection');
			Doc.element('specimenTubesTitle').innerHTML = "RNA tubes";
			Doc.element('specimenTubes').innerHTML = rnaTubes;
			Doc.element('specimenTubesMessageText').innerHTML = "The RNA tube(s) associated with this case.";

			Doc.hide('nofTubesSection');
			Doc.hide('arrivalDateSection');

			Doc.show('specimenTubesSection');
			Doc.show('samplingDateSection');
			
			Doc.hide('mutationAnalysisSection');
			Doc.hide('siteSection');
			Doc.hide('sampleTypeSection');
			Doc.hide('projectFocusSection');
			Doc.hide('otherPathNoteSection');

			thisCaseSamplingDate = thisCaseExtractionDate;
			// Populate 'Sampling date' field
			if (thisCaseSamplingDate != null)
			{
				frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
				persinfo.samplingDateTimeOnChange();
			}
		
			frm.plNumber.value = thisCasePlNumber;
			frm.pad.value = thisCasePAD;

			if (caseInfo.id != null && typeof caseInfo.id != 'undefined')
			{
				frm.samplingDate.disabled = true;
				frm.tubeContentType.disabled = true;
				frm.specimenType.disabled = true;
			}

			Wizard.setCurrentStep(3);
			if (updateMode)
			{
				Doc.show('goupdate');
			}
			else
			{
				Doc.show('goregister');
			}
			Doc.show('gocancel');
			frm.plNumber.focus();
		}
		else
		{
			Doc.hide('specimenTubesSection');	
			Wizard.setCurrentStep(3);
			Doc.show('gocancel');
			Doc.show('gonext');
		
			frm.arrivalDate.value = Dates.formatDate(new Date(), 'yyyyMMdd');

			frm.plNumber.focus();
			frm.nofTubes.setSelectionRange(0,1);
		}
	}

	persinfo.plNumberOnChange = function()
	{
		var frm = document.forms['meludi'];
		var plNumber = frm.plNumber.value;
		if (plNumber == '')
		{
			Wizard.setInputStatus('plNumber', 'warning', 'Missing');
			return;
		}
		frm.plNumber.value = plNumber.replace(/\+/, '-');
		Wizard.setInputStatus('plNumber', 'valid');
	}

	persinfo.padOnChange = function()
	{
		var frm = document.forms['meludi'];
		var PAD = frm.pad.value;
		if (PAD == '')
		{
			Wizard.setInputStatus('pad', 'warning', 'Missing');
			return;
		}
		frm.pad.value = PAD.replace(/\+/, '-');
		Wizard.setInputStatus('pad', 'valid');
	}

	persinfo.samplingDateTimeFocus = function(event)
	{
		if (event.keyCode == 13)
		{
			var focusTo = caseInfo.specimen && caseInfo.specimen.length > 0 ? 'mutationAnalysis' : 'reasonIfNoSpecimen'
			Doc.element(focusTo).focus();
		}
	}
	
	persinfo.nofTubesOnChange = function()
	{
		nofTubesIsValid = false;

		var frm = document.forms['meludi'];
		var nofTubes = frm.nofTubes.value;
		
		if (nofTubes == '')
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofTubes) < 1)
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Must be at least 1');
			return;		
		}
		Wizard.setInputStatus('nofTubes', 'valid');
		nofTubesIsValid = true; 
	}

	persinfo.samplingDateTimeOnChange = function()
	{
		var frm = document.forms['meludi'];
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;
		
		if (samplingDate == '')
		{
			Wizard.setInputStatus('samplingDate', 'warning', 'Missing date');
		}
		else
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Meludi.autoFillDate(samplingDate, true);
			frm.samplingDate.value = samplingDate;

			if (samplingDate != '' && !Dates.isDate(samplingDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}

			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
	}

	persinfo.validateStep3 = function(event)
	{
		var isValid = samplingDateIsValid;
		if (!isValid) event.preventDefault();
	}

	persinfo.initializeStep4 = function()
	{
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		var nofTubes = parseInt(frm.nofTubes.value);
		nofTubes = persinfo.fixNofTubes(tubeContentType, nofTubes);
		frm.nofTubes.value = nofTubes;
		var storageBoxSuffix = persinfo.fetchStorageBoxSuffix(tubeContentType);
		if (nofTubes > 2) Doc.addClass('step-2', 'auto-hide');
		
		var html = '';
		for (var i=0; i < nofTubes; i++)
		{
			var tubeContentTypeName = persinfo.fetchTubeContentTypeName(tubeContentType, i);
			var tubeContentTypeNameInternal = persinfo.fetchTubeContentTypeNameInternal(tubeContentType, tubeContentTypeName);
			var tubeName = persinfo.fetchTubeName(tubeContentType, caseInfo, i);
			var tubeNo = i+caseInfo.specimenFirstIndex;
			var tubeIdentifier = persinfo.fetchTubeIdentifier(tubeContentType, caseInfo, i);
			html += '<tr class="section-header">';
			html += '<td colspan="4">'+tubeContentTypeName+' Tube '+tubeName+'</td>';
			html += '</tr>';

			var boxMsg = 'Box-number where the '+tubeContentTypeNameInternal+' tube is located in.';
			var disabledAttribute = '';
			if (i == 0 && tubeContentType != 'RNA')
			{
				boxMsg = 'No storage space is allocated for first '+tubeContentTypeNameInternal+', since all amount is expected to be used for analysis.';
				disabledAttribute = ' disabled';
			}	
			html += '<tr>';
			html += '<td class="subprompt">Box</td>';
			html += '<td class="input">';
			html += '<input type="text" name="box.'+i+'" id="box.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'></td>';
			html += '<td class="status" id="box.'+i+'.status"></td>';
			html += '<td class="help"><span id="box.'+i+'.message" class="message"></span>'+boxMsg+'</td>';
			html += '</tr>';
			
			html += '<tr>';
			html += '<td class="subprompt">Row</td>';
			html += '<td class="input">';
			html += '<input type="text" name="row.'+i+'" id="row.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '&nbsp;Column<input type="text" name="column.'+i+'" id="column.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '</td>';
			html += '<td class="status" id="position.'+i+'.status"></td>';
			html += '<td class="help"><span id="position.'+i+'.message" class="message"></span></td>';		
			html += '</tr>';

			if (tubeContentType == 'Specimen')
			{
				html += '<tr>';
				html += '<td class="subprompt">Number of sections</td>';
				html += '<td class="input">';
				html += '<input type="text" name="nofSections.'+i+'" id="nofSections.'+i+'" style="width: 3em;" maxlength="2" value="1">';
				html += '</td>';
				html += '<td class="status" id="nofSections.'+i+'.status"></td>';
				html += '<td class="help"><span id="nofSections.'+i+'.message" class="message"></span>Number of sections (slices) in FFPE block.</td>';		
				html += '</tr>';
			}

			html += '<tr class="align-top">';
			html += '<td class="subprompt">Operator delivery comment</td>';
			html += '<td class="input"><textarea rows="3" name="comment.'+i+'" id="comment.'+i+'"></textarea></td>';
			html += '<td class="status" id="comment.'+i+'.status"></td>';
			html += '<td class="help"><span id="comment.'+i+'.message" class="message"></span>In this field, enter information specific for '+tubeContentTypeNameInternal+' tube ' +tubeIdentifier+'.</td>';		
			html += '</tr>';
		}
		Doc.element('specimen-tubes').innerHTML = html;

		Wizard.setCurrentStep(4);
		Doc.show('goregister');
		Doc.show('gocancel');
		frm['comment.0'].focus();
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=FindStoragePositions';	
		url += '&nofTubes='+frm.nofTubes.value;
		url += '&storageBoxSuffix='+storageBoxSuffix;
		
		Wizard.showLoadingAnimation('Looking for empty box positions...');
		Wizard.asyncJsonRequest(url, persinfo.storagePositionsLoaded);
	}
	
	persinfo.storagePositionsLoaded = function(response)
	{
		var freeWells = response.wells;
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		var nofTubes = parseInt(frm.nofTubes.value);

		// No storage space is allocated for first specimen,
		// since all amount is expected to be used for analysis.
		// Specimen tube with i == 0 is therefore skipped.
		var startIndex = 1;
		if (tubeContentType == 'RNA')
		{
			startIndex = 0;
		}
		for (var i = startIndex; i < nofTubes; i++)
		{
			var wellIndex = i-1;
			if (tubeContentType == 'RNA')
			{
				wellIndex = i;
			}
			var well = freeWells[wellIndex];
			if (!well || well == '') 
			{
				Wizard.setInputStatus('box.'+i, 'invalid', 'Missing box');
				return;
			}
			frm['box.'+i].value = well.bioPlate.name;
			frm['row.'+i].value = Meludi.wellToAlpha(well.row);
			frm['column.'+i].value = well.column+1;
			
			// Move focus to next field
			Events.addEventHandler('box.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'row.'+i });
			Events.addEventHandler('row.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'column.'+i });
			Events.addEventHandler('column.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'nofSections.'+i });
			Events.addEventHandler('nofSections.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'comment.'+i });
			
			// Form validation
			Events.addEventHandler('column.'+i, 'keypress', Events.integerOnly);
			Events.addEventHandler('box.'+i, 'change', persinfo.wellOnChange, { 'index': i });
			Events.addEventHandler('row.'+i, 'change', persinfo.wellOnChange, { 'index': i });
			Events.addEventHandler('column.'+i, 'change', persinfo.wellOnChange, { 'index': i });
			Events.addEventHandler('box.'+i, 'keyup', persinfo.boxOnKeyUp);
			Events.addEventHandler('nofSections.'+i, 'keypress', Events.integerOnly);
			Events.addEventHandler('nofSections.'+i, 'change', persinfo.nofSectionsOnChange, { 'index': i });
			
			wellIsValid[i] = true;
			Wizard.setInputStatus('position.'+i, 'valid');
			Wizard.setInputStatus('box.'+i, 'valid');
			if (tubeContentType == 'Specimen')
			{
				Wizard.setInputStatus('nofSections.'+i, 'valid');
			}
		}
	}

	persinfo.fixNofTubes = function(tubeContentType, nofTubes)
	{
		if (tubeContentType == 'DNA')
		{
			nofTubes = 1;
		}
		else if (tubeContentType == 'RNA')
		{
			nofTubes = 1;
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			nofTubes = 2;
		}
		return nofTubes;
	}
	
	persinfo.fetchStorageBoxSuffix = function(tubeContentType)
	{
		var storageBoxSuffix = null;
		if (tubeContentType == 'Specimen')
		{
			storageBoxSuffix = '_sp';
		}
		else if (tubeContentType == 'DNA')
		{
			storageBoxSuffix = null;
		}
		else if (tubeContentType == 'RNA')
		{
			storageBoxSuffix = '_rna';
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			storageBoxSuffix = '_rna';
		}
		return storageBoxSuffix;
	}
	
	persinfo.fetchTubeContentTypeName = function(tubeContentType, tubeIndex)
	{
		var tubeContentTypeName = tubeContentType;
		if (tubeContentType == 'DNA_RNA')
		{
			if (tubeIndex == 0)
			{
				tubeContentTypeName = 'DNA';
			}
			else if (tubeIndex == 1)
			{
				tubeContentTypeName = 'RNA';
			}
		}
		return tubeContentTypeName;
	}
	
	persinfo.fetchTubeContentTypeNameInternal = function(tubeContentType, tubeContentTypeName)
	{
		var tubeContentTypeNameInternal = tubeContentTypeName;
		// Convert all names except abbreviations to all lower case
		if (tubeContentType == 'Specimen')
		{
			tubeContentTypeNameInternal = tubeContentTypeName.toLowerCase();
		}
		return tubeContentTypeNameInternal;
	}
	
	persinfo.fetchTubeName = function(tubeContentType, caseInfo, tubeIndex)
	{
		var tubeName = Strings.encodeTags(caseInfo.originalName);
		if (tubeContentType == 'Specimen')
		{
			var tubeNo = tubeIndex + caseInfo.specimenFirstIndex;
			tubeName = tubeName+'.'+tubeNo;
		}
		else if (tubeContentType == 'DNA')
		{
			tubeName = tubeName+'.d';
		}
		else if (tubeContentType == 'RNA')
		{
			tubeName = tubeName+'.r';
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			if (tubeIndex == 0)
			{
				tubeName = tubeName+'.d';
			}
			else if (tubeIndex == 1)
			{
				tubeName = tubeName+'.r';
			}
		}
		return tubeName;
	}
	
	persinfo.fetchTubeIdentifier = function(tubeContentType, tubeName, tubeNo)
	{
		var tubeIdentifier = tubeName;
		if (tubeContentType == 'Specimen')
		{
			tubeIdentifier = tubeNo;
		}
		return tubeIdentifier;
	}
	
	persinfo.boxOnKeyUp = function(event)
	{
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		
		var index = Data.int(event.currentTarget, 'index');

		// Check box name
		var boxIsValid = true;
		var box = frm['box.'+index].value;
		Wizard.setInputStatus('box.'+index);
		// Check for prefix "ML_"
		if (box.indexOf('ML_') != 0)
		{
			boxIsValid = false;
		}
		var suffix = '_sp';
		if (tubeContentType == 'RNA')
		{
			suffix = '_rna';
		}
		// Check for suffix
		if (box.indexOf(suffix) != (box.length - suffix.length))
		{
			boxIsValid = false;
		}
		
		if (!boxIsValid)
		{
			Wizard.setInputStatus('box.'+index, 'invalid', 'Box name must start with "ML_" and end with "'+suffix+'"');
			return;
		}
	}
	
	persinfo.wellOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		var target = event.currentTarget;

		var index = Data.int(target, 'index');
		var box = frm['box.'+index].value;
		var row = frm['row.'+index].value;
		var column = frm['column.'+index].value;
		
		wellIsValid[index] = false;
		Wizard.setInputStatus('box.'+index);
		Wizard.setInputStatus('position.'+index);
		
		if (box == '') 
		{
			Wizard.setInputStatus('box.'+index, 'invalid', 'Missing box');
			return;
		}
		
		if (row == '')
		{
			Wizard.setInputStatus('position.'+index, 'invalid', 'Missing row');
			return;
		}

		if (column == '')
		{
			Wizard.setInputStatus('position.'+index, 'invalid', 'Missing column');
			return;
		}
		
		// Check for duplicates
		var nofTubes = parseInt(frm.nofTubes.value);
		for (var i=0; i < nofTubes; i++)
		{		
			if (i != index)
			{
				if (box == frm.elements['box.'+i].value && row == frm['row.'+i].value && column == frm['column.'+i].value)
				{
					Wizard.setInputStatus('position.'+index, 'invalid', 'The position is already used once in this wizard');				
					return;
				}
			}
		}

		Wizard.setInputStatus('position.'+index, 'checking', 'Checking...');
		
		// Seems to be ok -- check if used by another sample
		var request = Ajax.getXmlHttpRequest();
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=ValidateWell';
		url += '&box=' + encodeURIComponent(box);	
		url += '&row=' + encodeURIComponent(row);
		url += '&column=' + encodeURIComponent(column);
		
		Wizard.asyncJsonRequest(url, function(response) { persinfo.onBoxValidated(response, index) } );
	}
	
	persinfo.onBoxValidated = function(response, index)
	{
		Wizard.setInputStatus('position.'+index);
		if (response.message)
		{
			Wizard.setInputStatus(response.box ? 'box.'+index : 'position.'+index, 'invalid', Strings.encodeTags(response.message));
			return;
		}
		
		wellIsValid[index] = true;
		Wizard.setInputStatus('box.'+index, 'valid');
		Wizard.setInputStatus('position.'+index, 'valid');
	}

	persinfo.nofSectionsOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		var target = event.currentTarget;

		var index = Data.int(target, 'index');
		var nofSections = frm['nofSections.'+index].value;

		Wizard.setInputStatus('nofSections.'+index);

		if (nofSections == '')
		{
			Wizard.setInputStatus('nofSections'+index, 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofSections) < 1)
		{
			Wizard.setInputStatus('nofSections'+index, 'invalid', 'Must be at least 1');
			return;		
		}
		Wizard.setInputStatus('nofSections'+index, 'valid');
	}

	persinfo.validateStep4 = function(event)
	{
		var valid = false;

		// No storage space is allocated for first specimen,
		// since all amount is expected to be used for analysis.
		// Specimen tube with i == 0 is therefore skipped.
		for (var i = 1; i < wellIsValid.length; i++)
		{
			if (!wellIsValid[i])
			{
				event.preventDefault();
				return;
			}
		}
	}

	persinfo.submit = function()
	{
		var frm = document.forms['meludi'];
		var updateMode = caseInfo.id;

		caseInfo.plNumber = frm.plNumber.value;
		caseInfo.pad = frm.pad.value;
		caseInfo.samplingDate = Strings.trim(frm.samplingDate.value);
		caseInfo.tubeContentType = frm.tubeContentType.value;
		if (!updateMode)
		{
			caseInfo.arrivalDate = Strings.trim(frm.arrivalDate.value);
			caseInfo.mutationAnalysisKrasNras = frm.mutationAnalysisKrasNras.checked;
			caseInfo.mutationAnalysisBraf = frm.mutationAnalysisBraf.checked;
			caseInfo.mutationAnalysisEgfr = frm.mutationAnalysisEgfr.checked;
			caseInfo.site = frm.site.value;
			caseInfo.specimenType = frm.specimenType.value;
			caseInfo.projectFocus = frm.projectFocus.value;
			caseInfo.otherPathNote = frm.otherPathNote.value;
		
			var nofTubes = parseInt(frm.nofTubes.value);
			caseInfo.specimen = [];
			for (var i = 0; i < nofTubes; i++)
			{
				var specimenTube ={};
				specimenTube.name = persinfo.fetchTubeName(caseInfo.tubeContentType, caseInfo, i);

				specimenTube.box = frm['box.'+i].value;
				specimenTube.row = frm['row.'+i].value;
				specimenTube.column = frm['column.'+i].value;
				var nofSections = null;
				if (caseInfo.tubeContentType == 'Specimen')
				{
					nofSections = frm['nofSections.'+i].value;
				}
				specimenTube.nofSections = nofSections;
				specimenTube.comment = frm['comment.'+i].value;
				specimenTube.specimenType = frm.specimenType.value;
				caseInfo.specimen[i] = specimenTube;
			}
		}
		var submitInfo = {};
		submitInfo.patientInfo = patientInfo;
		submitInfo.caseInfo = caseInfo;

		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=' + (updateMode ? 'UpdateCase' : 'CreateCase');
		Wizard.showLoadingAnimation('Registering case...');
		Wizard.asyncJsonRequest(url, persinfo.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	persinfo.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return persinfo;
}();

Doc.onLoad(PersInfo.initPage);

